import * as React from 'react';
import React__default, { useContext, useRef, useEffect, useCallback, createContext, useMemo, useState, useReducer } from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import { Collapse, NavLink, NavItem, NavbarBrand } from 'react-bootstrap';

function _extends$1() {
  _extends$1 = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends$1.apply(this, arguments);
}

function _objectWithoutPropertiesLoose$1(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose$1(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var SelectableContext = /*#__PURE__*/React__default.createContext(null);
var makeEventKey = function makeEventKey(eventKey, href) {
  if (href === void 0) {
    href = null;
  }

  if (eventKey != null) return String(eventKey);
  return href || null;
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var ThemeContext = /*#__PURE__*/React__default.createContext({});
ThemeContext.Consumer;
    ThemeContext.Provider;

function useBootstrapPrefix(prefix, defaultPrefix) {
  var prefixes = useContext(ThemeContext);
  return prefix || prefixes[defaultPrefix] || defaultPrefix;
}

/**
 * Creates a `Ref` whose value is updated in an effect, ensuring the most recent
 * value is the one rendered with. Generally only required for Concurrent mode usage
 * where previous work in `render()` may be discarded befor being used.
 *
 * This is safe to access in an event handler.
 *
 * @param value The `Ref` value
 */

function useCommittedRef(value) {
  var ref = useRef(value);
  useEffect(function () {
    ref.current = value;
  }, [value]);
  return ref;
}

function useEventCallback(fn) {
  var ref = useCommittedRef(fn);
  return useCallback(function () {
    return ref.current && ref.current.apply(ref, arguments);
  }, [ref]);
}

var SidebarMenuContext = /*#__PURE__*/createContext({});

var _excluded$e = ["as", "children", "onClick", "bsPrefix", "className", "label"];
var propTypes$5 = {
  /** @default 'sidebar-menu-toggler' */
  bsPrefix: PropTypes.string,

  /** An accessible ARIA label for the toggler button. */
  label: PropTypes.string,

  /** @private */
  onClick: PropTypes.func,

  /**
   * The toggle content. When empty, the default toggle will be rendered.
   */
  children: PropTypes.node,
  as: PropTypes.elementType,
  className: PropTypes.string
};
var defaultProps$3 = {
  label: 'Toggle navigation'
};
var SidebarMenuToggle = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'button' : _ref$as,
      children = _ref.children,
      onClick = _ref.onClick,
      bsPrefix = _ref.bsPrefix,
      className = _ref.className,
      label = _ref.label,
      props = _objectWithoutProperties(_ref, _excluded$e);

  bsPrefix = useBootstrapPrefix(bsPrefix, 'sidebar-menu-toggle');

  var _ref2 = useContext(SidebarMenuContext) || {},
      onToggle = _ref2.onToggle,
      expanded = _ref2.expanded;

  var handleOnClick = useEventCallback(function (e) {
    onClick === null || onClick === void 0 ? void 0 : onClick(e);
    onToggle === null || onToggle === void 0 ? void 0 : onToggle();
  });

  if (Component === 'button') {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    props.type = 'button';
  }

  return /*#__PURE__*/React__default.createElement(Component, _extends$1({
    ref: ref
  }, props, {
    onClick: handleOnClick,
    className: classNames(className, bsPrefix, !expanded && 'collapsed'),
    "aria-label": label
  }), children || /*#__PURE__*/React__default.createElement("span", {
    className: "".concat(bsPrefix, "-icon")
  }));
});
SidebarMenuToggle.displayName = "SidebarMenuToggler";
SidebarMenuToggle.propTypes = propTypes$5;
SidebarMenuToggle.defaultProps = defaultProps$3;

/**
 * Safe chained function
 *
 * Will only create a new function if needed,
 * otherwise will pass back existing functions or null.
 *
 * @param {function} functions to chain
 * @returns {function|null}
 */
function createChainedFunction() {
  for (var _len = arguments.length, funcs = new Array(_len), _key = 0; _key < _len; _key++) {
    funcs[_key] = arguments[_key];
  }

  return funcs.filter(function (f) {
    return f != null;
  }).reduce(function (acc, f) {
    if (typeof f !== 'function') {
      throw new Error('Invalid Argument Type, must only provide functions, undefined, or null.');
    }

    if (acc === null) return f;
    return function chainedFunction() {
      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      // @ts-ignore
      acc.apply(this, args); // @ts-ignore

      f.apply(this, args);
    };
  }, null);
}

var _excluded$d = ["dimension", "getScrollValue", "children", "onEntering", "className", "bsPrefix"];
var propTypes$4 = {
  /** @default 'sidebar-menu-collapse' */
  bsPrefix: PropTypes.string,
  getScrollValue: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.func])
};
var SidebarMenuCollapse = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var _ref$dimension = _ref.dimension,
      dimension = _ref$dimension === void 0 ? "width" : _ref$dimension,
      _ref$getScrollValue = _ref.getScrollValue,
      getScrollValue = _ref$getScrollValue === void 0 ? function () {
    return 300;
  } : _ref$getScrollValue,
      children = _ref.children,
      onEntering = _ref.onEntering,
      className = _ref.className,
      initialBsPrefix = _ref.bsPrefix,
      props = _objectWithoutProperties(_ref, _excluded$d);

  var computedDimension = typeof dimension === 'function' ? dimension() : dimension;
  var sidebarMenuContext = useContext(SidebarMenuContext);
  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-collapse');
  var handleEntering = useMemo(function () {
    return createChainedFunction(function (elem) {
      if (typeof getScrollValue === 'function') {
        var scrollValue = getScrollValue(elem);
        elem.style[computedDimension] = typeof scrollValue === 'string' ? scrollValue : "".concat(scrollValue, "px");
      } else if (typeof getScrollValue === 'number') {
        elem.style[computedDimension] = "".concat(getScrollValue, "px");
      } else if (typeof getScrollValue === 'string') {
        elem.style[computedDimension] = getScrollValue;
      }
    }, onEntering);
  }, [computedDimension, getScrollValue, onEntering]);
  return /*#__PURE__*/React__default.createElement(Collapse, _extends$1({}, props, {
    dimension: dimension,
    onEntering: handleEntering,
    "in": !!(sidebarMenuContext !== null && sidebarMenuContext !== void 0 && sidebarMenuContext.expanded)
  }), /*#__PURE__*/React__default.createElement("div", {
    ref: ref,
    className: classNames(className, bsPrefix)
  }, children));
});
SidebarMenuCollapse.displayName = "SidebarMenuCollapse";
SidebarMenuCollapse.propTypes = propTypes$4;

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/**
 * Similar to invariant but only logs a warning if the condition is not met.
 * This can be used to log issues in development environments in critical
 * paths. Removing the logging code for production environments will keep the
 * same logic and follow the same code paths.
 */

var __DEV__ = process.env.NODE_ENV !== 'production';

var warning = function() {};

if (__DEV__) {
  var printWarning = function printWarning(format, args) {
    var len = arguments.length;
    args = new Array(len > 1 ? len - 1 : 0);
    for (var key = 1; key < len; key++) {
      args[key - 1] = arguments[key];
    }
    var argIndex = 0;
    var message = 'Warning: ' +
      format.replace(/%s/g, function() {
        return args[argIndex++];
      });
    if (typeof console !== 'undefined') {
      console.error(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) {}
  };

  warning = function(condition, format, args) {
    var len = arguments.length;
    args = new Array(len > 2 ? len - 2 : 0);
    for (var key = 2; key < len; key++) {
      args[key - 2] = arguments[key];
    }
    if (format === undefined) {
      throw new Error(
          '`warning(condition, format, ...args)` requires a warning ' +
          'message argument'
      );
    }
    if (!condition) {
      printWarning.apply(null, [format].concat(args));
    }
  };
}

var warning_1 = warning;

var NavContext = /*#__PURE__*/React__default.createContext(null);
NavContext.displayName = 'NavContext';

var _excluded$c = ["active", "className", "eventKey", "onSelect", "onClick", "as"];
var defaultProps$2 = {
  disabled: false
};
var AbstractNavItem = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var active = _ref.active,
      className = _ref.className,
      eventKey = _ref.eventKey,
      onSelect = _ref.onSelect,
      onClick = _ref.onClick,
      Component = _ref.as,
      props = _objectWithoutPropertiesLoose(_ref, _excluded$c);

  var navKey = makeEventKey(eventKey, props.href);
  var parentOnSelect = useContext(SelectableContext);
  var navContext = useContext(NavContext);
  var isActive = active;

  if (navContext) {
    if (!props.role && navContext.role === 'tablist') props.role = 'tab';
    var contextControllerId = navContext.getControllerId(navKey);
    var contextControlledId = navContext.getControlledId(navKey);
    process.env.NODE_ENV !== "production" ? warning_1(!contextControllerId || !props.id, "[react-bootstrap] The provided id '" + props.id + "' was overwritten by the current navContext with '" + contextControllerId + "'.") : void 0;
    process.env.NODE_ENV !== "production" ? warning_1(!contextControlledId || !props['aria-controls'], "[react-bootstrap] The provided aria-controls value '" + props['aria-controls'] + "' was overwritten by the current navContext with '" + contextControlledId + "'.") : void 0;
    props['data-rb-event-key'] = navKey;
    props.id = contextControllerId || props.id;
    props['aria-controls'] = contextControlledId || props['aria-controls'];
    isActive = active == null && navKey != null ? navContext.activeKey === navKey : active;
  }

  if (props.role === 'tab') {
    if (props.disabled) {
      props.tabIndex = -1;
      props['aria-disabled'] = true;
    }

    props['aria-selected'] = isActive;
  }

  var handleOnclick = useEventCallback(function (e) {
    if (onClick) onClick(e);
    if (navKey == null) return;
    if (onSelect) onSelect(navKey, e);
    if (parentOnSelect) parentOnSelect(navKey, e);
  });
  return /*#__PURE__*/React__default.createElement(Component, _extends({}, props, {
    ref: ref,
    onClick: handleOnclick,
    className: classNames(className, isActive && 'active')
  }));
});
AbstractNavItem.defaultProps = defaultProps$2;

var SidebarMenuNodeContext = /*#__PURE__*/React.createContext({});

var _excluded$b = ["bsPrefix", "as", "className", "eventKey", "active", "href", "disabled"];
var SidebarMenuNavLink = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      _ref$as = _ref.as,
      As = _ref$as === void 0 ? 'a' : _ref$as,
      className = _ref.className,
      eventKey = _ref.eventKey,
      active = _ref.active,
      href = _ref.href,
      disabled = _ref.disabled,
      props = _objectWithoutProperties(_ref, _excluded$b);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-nav-link');
  var nodeContext = useContext(SidebarMenuNodeContext);
  var navContext = useContext(NavContext);
  var navKey = makeEventKey(eventKey, href);
  var isActive = active == null && navKey != null ? navContext && navContext.activeKey === navKey : active;
  useEffect(function () {
    var _nodeContext$onActive;

    if (isActive) (_nodeContext$onActive = nodeContext.onActiveKeyChanged) === null || _nodeContext$onActive === void 0 ? void 0 : _nodeContext$onActive.call(nodeContext, navKey);
  }, [isActive, navKey, nodeContext]);
  return /*#__PURE__*/React__default.createElement(AbstractNavItem, _extends$1({
    ref: ref,
    as: As,
    href: href,
    active: active,
    eventKey: eventKey,
    disabled: disabled,
    className: classNames(className, bsPrefix, disabled && 'disabled')
  }, props));
});
SidebarMenuNavLink.displayName = "SidebarMenuNavLink";
SidebarMenuNavLink.propTypes = NavLink.propTypes;
SidebarMenuNavLink.defaultProps = NavLink.defaultProps;

var _excluded$a = ["bsPrefix"];
var SidebarMenuNavItem = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      props = _objectWithoutProperties(_ref, _excluded$a);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-nav-item');
  return /*#__PURE__*/React__default.createElement(NavItem, _extends$1({
    ref: ref,
    bsPrefix: bsPrefix
  }, props));
});
SidebarMenuNavItem.displayName = "SidebarMenuNavItem";
SidebarMenuNavItem.propTypes = NavItem.propTypes;
SidebarMenuNavItem.defaultProps = NavItem.defaultProps;

var rHyphen = /-(.)/g;
function camelize(string) {
  return string.replace(rHyphen, function (_, chr) {
    return chr.toUpperCase();
  });
}

var _excluded$9 = ["className", "bsPrefix", "as"];

var pascalCase = function pascalCase(str) {
  return str[0].toUpperCase() + camelize(str).slice(1);
};

// TODO: emstricten & fix the typing here! `createWithBsPrefix<TElementType>...`
function createWithBsPrefix(prefix, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$displayName = _ref.displayName,
      displayName = _ref$displayName === void 0 ? pascalCase(prefix) : _ref$displayName,
      Component = _ref.Component,
      defaultProps = _ref.defaultProps;

  var BsComponent = /*#__PURE__*/React__default.forwardRef(function (_ref2, ref) {
    var className = _ref2.className,
        bsPrefix = _ref2.bsPrefix,
        _ref2$as = _ref2.as,
        Tag = _ref2$as === void 0 ? Component || 'div' : _ref2$as,
        props = _objectWithoutPropertiesLoose(_ref2, _excluded$9);

    var resolvedPrefix = useBootstrapPrefix(bsPrefix, prefix);
    return /*#__PURE__*/React__default.createElement(Tag, _extends({
      ref: ref,
      className: classNames(className, resolvedPrefix)
    }, props));
  });
  BsComponent.defaultProps = defaultProps;
  BsComponent.displayName = displayName;
  return BsComponent;
}

var SidebarMenuNavIcon = createWithBsPrefix('sidebar-menu-nav-icon', {
  displayName: 'SidebarMenuNavIcon',
  Component: 'span'
});

var SidebarMenuNavTitle = createWithBsPrefix('sidebar-menu-nav-title', {
  displayName: 'SidebarMenuNavTitle',
  Component: 'span'
});

var _excluded$8 = ["as", "bsPrefix", "className"];
var propTypes$3 = {
  /** @default 'sidebar-menu-nav' */
  bsPrefix: PropTypes.string
};
var SidebarMenuNav = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var _ref$as = _ref.as,
      Component = _ref$as === void 0 ? "div" : _ref$as,
      initialBsPrefix = _ref.bsPrefix,
      className = _ref.className,
      props = _objectWithoutProperties(_ref, _excluded$8);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-nav');
  return /*#__PURE__*/React__default.createElement(Component, _extends$1({
    ref: ref
  }, props, {
    className: classNames(className, bsPrefix)
  }));
});
SidebarMenuNav.displayName = 'SidebarMenuNav';
SidebarMenuNav.propTypes = propTypes$3;
var SidebarMenuNav$1 = Object.assign(SidebarMenuNav, {
  Item: SidebarMenuNavItem,
  Link: SidebarMenuNavLink,
  Icon: SidebarMenuNavIcon,
  Title: SidebarMenuNavTitle
});

var _excluded$7 = ["bsPrefix"];
var SidebarMenuBrand = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      props = _objectWithoutProperties(_ref, _excluded$7);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-brand');
  return /*#__PURE__*/React__default.createElement(NavbarBrand, _extends$1({
    ref: ref,
    bsPrefix: bsPrefix
  }, props));
});
SidebarMenuBrand.displayName = "SidebarMenuBrand";

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/**
 * Use invariant() to assert state which your program assumes to be true.
 *
 * Provide sprintf-style format (only %s is supported) and arguments
 * to provide information about what broke and what you were
 * expecting.
 *
 * The invariant message will be stripped in production, but the invariant
 * will remain to ensure logic does not differ in production.
 */

process.env.NODE_ENV;

function defaultKey(key) {
  return 'default' + key.charAt(0).toUpperCase() + key.substr(1);
}

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function useUncontrolledProp(propValue, defaultValue, handler) {
  var wasPropRef = useRef(propValue !== undefined);

  var _useState = useState(defaultValue),
      stateValue = _useState[0],
      setState = _useState[1];

  var isProp = propValue !== undefined;
  var wasProp = wasPropRef.current;
  wasPropRef.current = isProp;
  /**
   * If a prop switches from controlled to Uncontrolled
   * reset its value to the defaultValue
   */

  if (!isProp && wasProp && stateValue !== defaultValue) {
    setState(defaultValue);
  }

  return [isProp ? propValue : stateValue, useCallback(function (value) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    if (handler) handler.apply(void 0, [value].concat(args));
    setState(value);
  }, [handler])];
}
function useUncontrolled(props, config) {
  return Object.keys(config).reduce(function (result, fieldName) {
    var _extends2;

    var _ref = result,
        defaultValue = _ref[defaultKey(fieldName)],
        propsValue = _ref[fieldName],
        rest = _objectWithoutPropertiesLoose(_ref, [defaultKey(fieldName), fieldName].map(_toPropertyKey));

    var handlerName = config[fieldName];

    var _useUncontrolledProp = useUncontrolledProp(propsValue, defaultValue, props[handlerName]),
        value = _useUncontrolledProp[0],
        handler = _useUncontrolledProp[1];

    return _extends({}, rest, (_extends2 = {}, _extends2[fieldName] = value, _extends2[handlerName] = handler, _extends2));
  }, props);
}

var context$1 = /*#__PURE__*/React.createContext({});
context$1.displayName = 'SidebarMenuSubContext';

var context = /*#__PURE__*/React__default.createContext(null);
context.displayName = 'NavbarContext';

var _excluded$6 = ["bsPrefix", "className", "children", "label", "as", "onClick"];
var defaultProps$1 = {
  label: 'Toggle navigation'
};
var NavbarToggle = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var bsPrefix = _ref.bsPrefix,
      className = _ref.className,
      children = _ref.children,
      label = _ref.label,
      _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'button' : _ref$as,
      onClick = _ref.onClick,
      props = _objectWithoutPropertiesLoose(_ref, _excluded$6);

  bsPrefix = useBootstrapPrefix(bsPrefix, 'navbar-toggler');

  var _ref2 = useContext(context) || {},
      onToggle = _ref2.onToggle,
      expanded = _ref2.expanded;

  var handleClick = useEventCallback(function (e) {
    if (onClick) onClick(e);
    if (onToggle) onToggle();
  });

  if (Component === 'button') {
    props.type = 'button';
  }

  return /*#__PURE__*/React__default.createElement(Component, _extends({}, props, {
    ref: ref,
    onClick: handleClick,
    "aria-label": label,
    className: classNames(className, bsPrefix, !expanded && 'collapsed')
  }), children || /*#__PURE__*/React__default.createElement("span", {
    className: bsPrefix + "-icon"
  }));
});
NavbarToggle.displayName = 'NavbarToggle';
NavbarToggle.defaultProps = defaultProps$1;

var _excluded$5 = ["bsPrefix", "onClick", "className", "as", "children", "label"];
var SidebarMenuSubToggle = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      onClick = _ref.onClick,
      className = _ref.className,
      _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'button' : _ref$as,
      children = _ref.children,
      label = _ref.label,
      props = _objectWithoutProperties(_ref, _excluded$5);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-sub-toggle');

  var _useContext = useContext(context$1),
      activeKey = _useContext.activeKey,
      onSelect = _useContext.onSelect,
      onToggle = _useContext.onToggle,
      expanded = _useContext.expanded;

  var _useContext2 = useContext(context$1),
      eventKey = _useContext2.eventKey;

  var handleOnClick = useMemo(function () {
    return function () {
      var eventKeyPassed = eventKey === activeKey ? null : eventKey;
      onSelect === null || onSelect === void 0 ? void 0 : onSelect(eventKeyPassed);
    };
  }, [eventKey, activeKey, onSelect]);
  var handleClick = useEventCallback(function (e) {
    onClick === null || onClick === void 0 ? void 0 : onClick(e);
    onToggle === null || onToggle === void 0 ? void 0 : onToggle();
    handleOnClick();
  });

  if (Component === 'button') {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    props.type = 'button';
  }

  return /*#__PURE__*/React__default.createElement(Component, _extends$1({}, props, {
    ref: ref,
    onClick: handleClick,
    "aria-label": label,
    className: classNames(className, bsPrefix, !expanded && 'collapsed')
  }), children || /*#__PURE__*/React__default.createElement("span", {
    className: "".concat(bsPrefix, "-icon")
  }));
});
SidebarMenuSubToggle.displayName = "SidebarMenuSubToggle";
SidebarMenuSubToggle.propTypes = NavbarToggle.propTypes;
SidebarMenuSubToggle.defaultProps = NavbarToggle.defaultProps;

var _excluded$4 = ["children", "bsPrefix", "className"];
var propTypes$2 = {
  /** @default 'sidebar-menu-sub-collapse' */
  bsPrefix: PropTypes.string
};
var SidebarMenuSubCollapse = /*#__PURE__*/React.forwardRef(function (_ref, ref) {
  var children = _ref.children,
      initialBsPrefix = _ref.bsPrefix,
      className = _ref.className,
      props = _objectWithoutProperties(_ref, _excluded$4);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-sub-collapse');
  var subContext = useContext(context$1);
  return /*#__PURE__*/React.createElement(Collapse, _extends$1({
    "in": !!(subContext && subContext.expanded)
  }, props), /*#__PURE__*/React.createElement("div", {
    ref: ref,
    className: classNames(bsPrefix, className)
  }, children));
});
SidebarMenuSubCollapse.displayName = 'SidebarMenuSubCollapse';
SidebarMenuSubCollapse.propTypes = propTypes$2;

var _excluded$3 = ["with", "onActiveKeyChanged"],
    _excluded2$2 = ["activeNodeKey", "onNodeSelect"];

var SidebarMenuNode = function SidebarMenuNode(_ref) {
  var With = _ref["with"],
      onActiveKeyChanged = _ref.onActiveKeyChanged,
      props = _objectWithoutProperties(_ref, _excluded$3);

  var _useUncontrolled = useUncontrolled(props, {
    activeNodeKey: 'onNodeSelect'
  }),
      activeNodeKey = _useUncontrolled.activeNodeKey,
      onNodeSelect = _useUncontrolled.onNodeSelect,
      controlledProps = _objectWithoutProperties(_useUncontrolled, _excluded2$2);

  var nodeContextValue = useMemo(function () {
    return {
      onActiveKeyChanged: onActiveKeyChanged,
      onSelect: onNodeSelect,
      activeKey: activeNodeKey
    };
  }, [activeNodeKey, onActiveKeyChanged, onNodeSelect]);
  return /*#__PURE__*/React__default.createElement(SidebarMenuNodeContext.Provider, {
    value: nodeContextValue
  }, /*#__PURE__*/React__default.createElement(With, controlledProps));
};

var _excluded$2 = ["bsPrefix", "as", "eventKey", "className"],
    _excluded2$1 = ["expanded", "onToggle"];
var propTypes$1 = {
  /**
   * Set a custom element for this component.
   */
  as: PropTypes.elementType,

  /**
   * A callback fired when the `<SidebarMenuSub>` body collapses or expands. Fired when
   * a `<SidebarMenuSub.Toggle>` is clicked and called with the new `expanded`
   * boolean value.
   *
   * @controllable expanded
   */
  onToggle: PropTypes.func,

  /**
   * Controls the visiblity of the submenu body
   *
   * @controllable onToggle
   */
  expanded: PropTypes.bool,

  /**
   * The ARIA role for the SidebarMenuSub.
   *
   * @default 'navigation'
   */
  role: PropTypes.string,

  /**
   * 
   */
  defaultExpanded: PropTypes.bool
};
var SidebarMenuSub = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'div' : _ref$as,
      eventKey = _ref.eventKey,
      className = _ref.className,
      props = _objectWithoutProperties(_ref, _excluded$2);

  var _useUncontrolled = useUncontrolled(props, {
    expanded: 'onToggle'
  }),
      expanded = _useUncontrolled.expanded,
      _onToggle = _useUncontrolled.onToggle,
      controlledProps = _objectWithoutProperties(_useUncontrolled, _excluded2$1);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu-sub');

  var _useContext = useContext(SidebarMenuNodeContext),
      parentNodeActiveKey = _useContext.activeKey,
      onParentSelect = _useContext.onSelect,
      onParentActiveKeyChanged = _useContext.onActiveKeyChanged;

  var _useContext2 = useContext(SidebarMenuContext),
      exclusiveExpand = _useContext2.exclusiveExpand;

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      activeKey = _useState2[0],
      setActiveKey = _useState2[1];

  var navContext = useContext(NavContext);
  var currentActiveKey = navContext === null || navContext === void 0 ? void 0 : navContext.activeKey;
  var subContextValue = useMemo(function () {
    return {
      bsPrefix: bsPrefix,
      eventKey: eventKey,
      onSelect: onParentSelect,
      activeKey: parentNodeActiveKey,
      onToggle: function onToggle() {
        return _onToggle === null || _onToggle === void 0 ? void 0 : _onToggle(!expanded);
      },
      onActiveKeyChanged: onParentActiveKeyChanged,
      expanded: exclusiveExpand && typeof eventKey !== 'undefined' ? eventKey === parentNodeActiveKey : !!expanded
    };
  }, [bsPrefix, eventKey, exclusiveExpand, expanded, onParentSelect, _onToggle, parentNodeActiveKey, onParentActiveKeyChanged]);
  var onActiveKeyChanged = useMemo(function () {
    return createChainedFunction(function (eventKey) {
      return setActiveKey(eventKey);
    }, onParentActiveKeyChanged);
  }, [onParentActiveKeyChanged]);
  var activeKeyChangedCallback = useMemo(function () {
    return function (currentActiveKey) {
      if (currentActiveKey && currentActiveKey === activeKey) {
        exclusiveExpand ? onParentSelect === null || onParentSelect === void 0 ? void 0 : onParentSelect(eventKey) : _onToggle === null || _onToggle === void 0 ? void 0 : _onToggle(true);
      }
    };
  }, [activeKey, eventKey, exclusiveExpand, onParentSelect, _onToggle]);
  useEffect(function () {
    activeKeyChangedCallback(currentActiveKey);
  }, [currentActiveKey, activeKeyChangedCallback]);
  return /*#__PURE__*/React__default.createElement(context$1.Provider, {
    value: subContextValue
  }, /*#__PURE__*/React__default.createElement(SidebarMenuNode, _extends$1({
    "with": Component,
    ref: ref,
    onActiveKeyChanged: onActiveKeyChanged,
    className: classNames(className, bsPrefix)
  }, controlledProps)));
});
SidebarMenuSub.displayName = "SidebarMenuSub";
SidebarMenuSub.propTypes = propTypes$1;
var SidebarMenuSub$1 = Object.assign(SidebarMenuSub, {
  Collapse: SidebarMenuSubCollapse,
  Toggle: SidebarMenuSubToggle
});

var SidebarMenuHeader = createWithBsPrefix('sidebar-menu-header', {
  displayName: 'SidebarMenuHeader',
  Component: 'div'
});

var SidebarMenuBody = createWithBsPrefix('sidebar-menu-body', {
  displayName: 'SidebarMenuBody',
  Component: 'div'
});

var SidebarMenuFooter = createWithBsPrefix('sidebar-menu-footer', {
  displayName: 'SidebarMenuFooter',
  Component: 'div'
});

var SidebarMenuText = createWithBsPrefix('sidebar-menu-text', {
  displayName: 'SidebarMenuText',
  Component: 'span'
});

var toArray = Function.prototype.bind.call(Function.prototype.call, [].slice);
/**
 * Runs `querySelectorAll` on a given element.
 * 
 * @param element the element
 * @param selector the selector
 */

function qsa(element, selector) {
  return toArray(element.querySelectorAll(selector));
}

/**
 * Returns a function that triggers a component update. the hook equivalent to
 * `this.forceUpdate()` in a class component. In most cases using a state value directly
 * is preferable but may be required in some advanced usages of refs for interop or
 * when direct DOM manipulation is required.
 *
 * ```ts
 * const forceUpdate = useForceUpdate();
 *
 * const updateOnClick = useCallback(() => {
 *  forceUpdate()
 * }, [forceUpdate])
 *
 * return <button type="button" onClick={updateOnClick}>Hi there</button>
 * ```
 */

function useForceUpdate() {
  // The toggling state value is designed to defeat React optimizations for skipping
  // updates when they are stricting equal to the last state value
  var _useReducer = useReducer(function (state) {
    return !state;
  }, false),
      dispatch = _useReducer[1];

  return dispatch;
}

var toFnRef = function toFnRef(ref) {
  return !ref || typeof ref === 'function' ? ref : function (value) {
    ref.current = value;
  };
};

function mergeRefs(refA, refB) {
  var a = toFnRef(refA);
  var b = toFnRef(refB);
  return function (value) {
    if (a) a(value);
    if (b) b(value);
  };
}
/**
 * Create and returns a single callback ref composed from two other Refs.
 *
 * ```tsx
 * const Button = React.forwardRef((props, ref) => {
 *   const [element, attachRef] = useCallbackRef<HTMLButtonElement>();
 *   const mergedRef = useMergedRefs(ref, attachRef);
 *
 *   return <button ref={mergedRef} {...props}/>
 * })
 * ```
 *
 * @param refA A Callback or mutable Ref
 * @param refB A Callback or mutable Ref
 * @category refs
 */

function useMergedRefs(refA, refB) {
  return useMemo(function () {
    return mergeRefs(refA, refB);
  }, [refA, refB]);
}

var TabContext = /*#__PURE__*/React__default.createContext(null);

var _excluded$1 = ["as", "onSelect", "activeKey", "role", "onKeyDown"];

// eslint-disable-next-line @typescript-eslint/no-empty-function
var noop = function noop() {};

var AbstractNav = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'ul' : _ref$as,
      onSelect = _ref.onSelect,
      activeKey = _ref.activeKey,
      role = _ref.role,
      onKeyDown = _ref.onKeyDown,
      props = _objectWithoutPropertiesLoose(_ref, _excluded$1);

  // A ref and forceUpdate for refocus, b/c we only want to trigger when needed
  // and don't want to reset the set in the effect
  var forceUpdate = useForceUpdate();
  var needsRefocusRef = useRef(false);
  var parentOnSelect = useContext(SelectableContext);
  var tabContext = useContext(TabContext);
  var getControlledId, getControllerId;

  if (tabContext) {
    role = role || 'tablist';
    activeKey = tabContext.activeKey;
    getControlledId = tabContext.getControlledId;
    getControllerId = tabContext.getControllerId;
  }

  var listNode = useRef(null);

  var getNextActiveChild = function getNextActiveChild(offset) {
    var currentListNode = listNode.current;
    if (!currentListNode) return null;
    var items = qsa(currentListNode, '[data-rb-event-key]:not(.disabled)');
    var activeChild = currentListNode.querySelector('.active');
    if (!activeChild) return null;
    var index = items.indexOf(activeChild);
    if (index === -1) return null;
    var nextIndex = index + offset;
    if (nextIndex >= items.length) nextIndex = 0;
    if (nextIndex < 0) nextIndex = items.length - 1;
    return items[nextIndex];
  };

  var handleSelect = function handleSelect(key, event) {
    if (key == null) return;
    if (onSelect) onSelect(key, event);
    if (parentOnSelect) parentOnSelect(key, event);
  };

  var handleKeyDown = function handleKeyDown(event) {
    if (onKeyDown) onKeyDown(event);
    var nextActiveChild;

    switch (event.key) {
      case 'ArrowLeft':
      case 'ArrowUp':
        nextActiveChild = getNextActiveChild(-1);
        break;

      case 'ArrowRight':
      case 'ArrowDown':
        nextActiveChild = getNextActiveChild(1);
        break;

      default:
        return;
    }

    if (!nextActiveChild) return;
    event.preventDefault();
    handleSelect(nextActiveChild.dataset.rbEventKey, event);
    needsRefocusRef.current = true;
    forceUpdate();
  };

  useEffect(function () {
    if (listNode.current && needsRefocusRef.current) {
      var activeChild = listNode.current.querySelector('[data-rb-event-key].active');
      if (activeChild) activeChild.focus();
    }

    needsRefocusRef.current = false;
  });
  var mergedRef = useMergedRefs(ref, listNode);
  return /*#__PURE__*/React__default.createElement(SelectableContext.Provider, {
    value: handleSelect
  }, /*#__PURE__*/React__default.createElement(NavContext.Provider, {
    value: {
      role: role,
      // used by NavLink to determine it's role
      activeKey: makeEventKey(activeKey),
      getControlledId: getControlledId || noop,
      getControllerId: getControllerId || noop
    }
  }, /*#__PURE__*/React__default.createElement(Component, _extends({}, props, {
    onKeyDown: handleKeyDown,
    ref: mergedRef,
    role: role
  }))));
});

var _excluded = ["bsPrefix", "collapseOnSelect", "exclusiveExpand", "className", "variant", "expand", "hide", "bg", "rtl", "as"],
    _excluded2 = ["expanded", "onSelect", "activeKey", "onToggle"];
var propTypes = {
  /** @default 'sidebar-menu' */
  bsPrefix: PropTypes.string,

  /**
   * The general visual variant of the SidebarMenu.
   * Use in combination with the `bg` prop, `background-color` utilities,
   * or your own background styles.
   *
   * @type {('light'|'dark')}
   */
  variant: PropTypes.string,

  /**
   * RTL direction.
   * 
   * @default false
   */
  rtl: PropTypes.bool,

  /**
   * The breakpoint, below which, the SidebarMenu will collapse.
   * When `true` the SidebarMenu will always be expanded regardless of screen size.
   */
  expand: PropTypes.oneOf([true, 'sm', 'md', 'lg', 'xl', 'xxl']).isRequired,

  /**
  * The breakpoint, below which, the SidebarMenu will hide.
  * When `true` the SidebarMenu will always be hidden regardless of screen size.
  * 
  * * @default false
  */
  hide: PropTypes.oneOf([true, 'sm', 'md', 'lg', 'xl', 'xxl']).isRequired,

  /**
   * Controls the visiblity of the sidebar body
   *
   * @controllable onToggle
   */
  expanded: PropTypes.bool,

  /**
   * A convenience prop for adding `bg-*` utility classes since they are so commonly used here.
   * `light` and `dark` are common choices but any `bg-*` class is supported, including any custom ones you might define.
   */
  bg: PropTypes.string,

  /**
   * Set a custom element for this component.
   */
  as: PropTypes.elementType,

  /**
   * A callback fired when the `<SidebarMenu>` body collapses or expands. Fired when
   * a `<SidebarMenu.Toggle>` is clicked and called with the new `expanded`
   * boolean value.
   *
   * @controllable expanded
   */
  onToggle: PropTypes.func,

  /**
   * A callback fired when a descendant of a child `<SidebarMenuNav>` is selected. Should
   * be used to execute complex closing or other miscellaneous actions desired
   * after selecting a descendant of `<SidebarMenuNav>`. Does nothing if no `<SidebarMenuNav>` or `<SidebarMenuNav>`
   * descendants exist. The callback is called with an eventKey, which is a
   * prop from the selected `<SidebarMenuNav>` descendant, and an event.
   *
   * ```js
   * function (
   *  eventKey: mixed,
   *  event?: SyntheticEvent
   * )
   * ```
   *
   * For basic closing behavior after all `<SidebarMenuNav>` descendant onSelect events in
   * mobile viewports, try using collapseOnSelect.
   *
   * Note: If you are manually closing the sidebar using this `OnSelect` prop,
   * ensure that you are setting `expanded` to false and not *toggling* between
   * true and false.
   */
  onSelect: PropTypes.func,

  /**
   * Toggles `expanded` to `false` after the onSelect event of a descendant of a
   * child `<SidebarMenu>` fires. Does nothing if no `<SidebarMenu>` or `<SidebarMenu>` descendants exist.
   *
   * Manually controlling `expanded` via the onSelect callback is recommended instead,
   * for more complex operations that need to be executed after
   * the `select` event of `<SidebarMenu>` descendants.
   */
  collapseOnSelect: PropTypes.bool,

  /**
   * Only allow one `<SidebarMenu.Sub.Collapse>` open at a time.
   */
  exclusiveExpand: PropTypes.bool,

  /**
   * The ARIA role for the sidebar, will default to 'navigation' for
   * SidebarMenu whose `as` is something other than `<nav>`.
   *
   * @default 'navigation'
   */
  role: PropTypes.string,

  /**
   * Marks the SidebarMenuNavItem with a matching `eventKey` (or `href` if present) as active.
   */
  activeKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number])
};
var defaultProps = {
  rtl: false,
  expand: true,
  hide: false,
  variant: 'light',
  collapseOnSelect: false,
  defaultExpanded: true
};
var SidebarMenu = /*#__PURE__*/React__default.forwardRef(function (_ref, ref) {
  var initialBsPrefix = _ref.bsPrefix,
      collapseOnSelect = _ref.collapseOnSelect,
      exclusiveExpand = _ref.exclusiveExpand,
      className = _ref.className,
      variant = _ref.variant,
      expand = _ref.expand,
      hide = _ref.hide,
      bg = _ref.bg,
      rtl = _ref.rtl,
      _ref$as = _ref.as,
      Component = _ref$as === void 0 ? 'aside' : _ref$as,
      props = _objectWithoutProperties(_ref, _excluded);

  var _useUncontrolled = useUncontrolled(props, {
    expanded: 'onToggle',
    activeKey: 'onSelect'
  }),
      expanded = _useUncontrolled.expanded,
      onSelect = _useUncontrolled.onSelect,
      activeKey = _useUncontrolled.activeKey,
      _onToggle = _useUncontrolled.onToggle,
      controlledProps = _objectWithoutProperties(_useUncontrolled, _excluded2);

  var bsPrefix = useBootstrapPrefix(initialBsPrefix, 'sidebar-menu');
  var handleSelect = useCallback(function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    onSelect === null || onSelect === void 0 ? void 0 : onSelect.apply(void 0, args);

    if (collapseOnSelect && expanded) {
      _onToggle === null || _onToggle === void 0 ? void 0 : _onToggle(false);
    }
  }, [onSelect, collapseOnSelect, expanded, _onToggle]);

  if (controlledProps.role === undefined && Component !== 'nav') {
    controlledProps.role = 'navigation';
  }

  var expandClass = "".concat(bsPrefix, "-expand");

  if (typeof expand === 'string') {
    expandClass = "".concat(expandClass, "-").concat(expand);
  }

  var hideClass = "".concat(bsPrefix, "-hide");

  if (typeof hide === 'string') {
    hideClass = "".concat(hideClass, "-").concat(hide);
  }

  var sidebarMenuContextValue = useMemo(function () {
    return {
      rtl: !!rtl,
      expanded: !!expanded,
      onToggle: function onToggle() {
        return _onToggle === null || _onToggle === void 0 ? void 0 : _onToggle(!expanded);
      },
      exclusiveExpand: exclusiveExpand
    };
  }, [expanded, _onToggle, rtl, exclusiveExpand]);
  return /*#__PURE__*/React__default.createElement(SidebarMenuContext.Provider, {
    value: sidebarMenuContextValue
  }, /*#__PURE__*/React__default.createElement(SelectableContext.Provider, {
    value: handleSelect
  }, /*#__PURE__*/React__default.createElement(SidebarMenuNode, _extends$1({
    "with": AbstractNav,
    as: Component,
    ref: ref,
    activeKey: activeKey,
    className: classNames(className, bsPrefix, hide && hideClass, expand && expandClass, expanded && 'show', rtl && "".concat(bsPrefix, "-rtl"), variant && "".concat(bsPrefix, "-").concat(variant), bg && "bg-".concat(bg))
  }, controlledProps))));
});
SidebarMenu.displayName = "SidebarMenu";
SidebarMenu.defaultProps = defaultProps;
SidebarMenu.propTypes = propTypes;
var SidebarMenu$1 = Object.assign(SidebarMenu, {
  Nav: SidebarMenuNav$1,
  Sub: SidebarMenuSub$1,
  Brand: SidebarMenuBrand,
  Collapse: SidebarMenuCollapse,
  Toggle: SidebarMenuToggle,
  Header: SidebarMenuHeader,
  Body: SidebarMenuBody,
  Footer: SidebarMenuFooter,
  Text: SidebarMenuText
});

export default SidebarMenu$1;
export { SidebarMenuBody, SidebarMenuBrand, SidebarMenuCollapse, SidebarMenuContext, SidebarMenuFooter, SidebarMenuHeader, SidebarMenuNav$1 as SidebarMenuNav, SidebarMenuNavIcon, SidebarMenuNavItem, SidebarMenuNavLink, SidebarMenuNavTitle, SidebarMenuSub$1 as SidebarMenuSub, context$1 as SidebarMenuSubContext };
